#!/bin/bash
# psvutil.sh for Linux (Ubuntu)
SCRIPTPATH=`dirname $0`
# Change working dir
cd ${SCRIPTPATH}

. ./common.sh

# Get os release
QN_OS_ID=`cat /etc/os-release | sed -n -e 's/^ID=//p' | sed -s "s/^\(\(\"\(.*\)\"\)\|\('\(.*\)'\)\)\$/\\3\\5/g"`
# Get OS major ver
QN_OS_VER_ID=`cat /etc/os-release | sed -n -e 's/^VERSION_ID=//p' | sed -s "s/^\(\(\"\(.*\)\"\)\|\('\(.*\)'\)\)\$/\\3\\5/g"`

resources() {
    qn_write header ${_QN_RES_FILE} resources
	local list=`cat /proc/mounts | grep -E '/dev/sd|/run/media' | awk '{ print $1}'`
    IFS=$'\n'
    i=0
    _usbN=1
    rootVar=0;
    for item in $list
        do
            device=$(df -B 1 | grep $item)
            path=`echo $device | awk '{print $6}'`
            name=`basename ${path}`

                case ${name} in
                    /run/media/*)
                        :
                    ;;
                    [/])
                        if ((rootVar > 0)); then
                            continue
                        else
                            name="Root file system";
                            rootVar=$((rootVar+1))
                        fi
                    ;;
                     "*")
                            continue
                     ;;
                esac
		size=$(echo $device | awk '{print $2}')
		free=$(echo $device | awk '{print $4}')

		qn_write element ${_QN_RES_FILE} $i $name $size $free

        i=$((i+1))
    done
    qn_write footer ${_QN_RES_FILE}
}

folders() {
    GetSharedFolders $1 $2
}

# Get environment status and report. For now it is only firewall status.
getHealthTest(){

   if (/usr/bin/firewall-cmd -q --state); then
       # Get firewall active zone.
       activeFirewallZone=$(firewall-cmd --get-active-zones | awk 'NR==1{print $1}')
       # Add fileflex ports to active zone. Ports are added to running not parmanent.
       # parmanent config is added from package installer
       if ! (/usr/bin/firewall-cmd --zone=$activeFirewallZone --add-port=4008-4009/tcp); then
           ReportHealthError "${QN_ERR_CODE_FIREWALL}" "Unable to open firewall ports"
       fi
       /usr/bin/firewall-cmd --reload
       ReportHealthNormal
   else
       # If firewall not running report health normal
       ReportHealthNormal
   fi
}


qn_main_exec $1 $2
