/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 9.2.2018
 * Time: 11:43
 * Author Petar Vanchev (petarv@qnext.com)
 * Author Milen Dimitrov (milend@qnext.com)
 */

/**
 * Activation frame
 *
 * @constructor
 */
var FrameActivate = function () {
    var options = {};
    options["canSendLogs"] = true;
    options["hasHashListener"] = true;

    FrameEncrypt.call(this, FrameActivate.VID, $('body'), options);
};

QnUtil.inherits(FrameActivate, FrameEncrypt);
QnUtil.extend(FrameActivate.prototype, {

    /* Public method */

    updateFrame: function () {
        var hash = location.hash && location.hash.charAt(0) === "#" ? location.hash.substring(1) : null;
        var page;
        if (hash) {
            // If there is a hash - use it
            page = hash;
        } else if (getGlobal(KEY_GLOBAL_CODE_ACTIVATE)) {
            // If no hash - check is there code, then use it
            page = FrameActivate._PAGE_ACTIVATE;
        } else {
            // Finally fallback to home page
            page = FrameActivate._PAGE_HOME;
        }
        log("Change page: " + page);
        var isActivate = page === FrameActivate._PAGE_ACTIVATE;
        $("#pageHome").toggle(!isActivate);
        $("#pageActivate").toggle(isActivate);
        if (isActivate)
            this._startActivate();

        // home page
        setTextContext("label_remote_access");
        setTextContext("label_remote_activate");
        // set tooltip text to activate page
        setTextContext("field_code_info");

        FrameActivate.superclass.updateFrame.apply(this);
    },

    /* Protected method */

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getFieldIdList: function () {
        var list = FrameActivate.superclass._getFieldIdList.call(this);
        list.push(FrameWithForm.FIELD_CODE);
        return list;
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getBtnIdList: function () {
        return [BTN_ACTIVATE_ID, BTN_CANCEL_ID];
    },

    /**
     *
     * @inheritDoc
     * @private
     */
    _checkCanSubmitImpl: function () {
        if(this._hasEncComp())
            return this._compEncryption.checkEncryption(true);

        return FrameActivate.superclass._checkCanSubmitImpl.apply(this);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitImpl: function (event) {
        var code = this._getFormControl(FrameWithForm.FIELD_CODE).val();
        var toSubmit = false;
        //  Validate activation code.
        //  See "device.short.code.length" prop in server
        //  Old method _validateCode
        if (code.length < 6) {
            this._setControlsState(false);
            showMessage(ERROR_INVALID_CODE, "error_code_short");
        } else {
            // FF-6200 and FF-6336 is in parent/super class
            toSubmit = FrameActivate.superclass._handleSubmitImpl.apply(this);
        }
        return toSubmit;
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _fillSubmitParams: function (params) {
        FrameActivate.superclass._fillSubmitParams.call(this, params);
        params["cmd"] = "add";
        params["user"] = this._getFormControl(FrameWithForm.FIELD_USERNAME).val().trim();
        params["user-password"] = this._getFormControl(FrameWithForm.FIELD_PASSWORD).val().trim();
        params["user-domain"] = this._getFormControl(FrameWithForm.FIELD_DOMAIN).val().trim();
        params["code"] = this._getFormControl(FrameWithForm.FIELD_CODE).val().trim();

        var mspId = getGlobal(KEY_GLOBAL_MSP_ID);
        if (mspId)
            params["mspId"] = mspId;
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitSuccessImpl: function (resp, requestParams) {
        if (resp.result) {
            this._handleActivationSuccess(resp["redirectURL"]);
            log("RegisterCode success with " + requestParams["code"]);
        } else {
            log("RegisterCode failure with " + requestParams["code"]);
            if(this._hasEncComp())
                this._compEncryption.updateEncComps();
        }
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitFailImpl: function (xhr, err) {
        if (xhr.responseText) {
            try {
                var response = JSON.parse(xhr.responseText);
                var error = response["error"];
                log("Code register failed error code: " + error["code"]);
            } catch (e) {
                log("Code register failed: " + e);
            }
        }
        if(this._hasEncComp())
            this._compEncryption.updateEncComps();
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleActionImpl: function(action, event) {
        switch (action) {
            case "openRegister":
                this._onRegister();
                break;
            case "openActivate":
                this._onActivate();
                break;
            default:
                FrameActivate.superclass._handleActionImpl.call(this, action, event);
        }
    },

    /**
     * @inheritDoc
     * @protected
     */
    _handleCancel: function () {
       var cancelLink = getGlobal(KEY_GLOBAL_CANCEL_LINK);
       if (cancelLink) {
           window.open(cancelLink, '_self');
       } else
           FrameActivate.superclass._handleCancel.call(this);
    },

    /**
     * @inheritDoc
     * @protected
     */
    _initContentImpl: function ($cont) {
        document.title = QnLocale.getStr("activate_app_title");
        this._renderPageHome($cont);
        var  $formCont = $('<div id="pageActivate" style="display: none;"></div>').appendTo($cont);
        FrameActivate.superclass._initContentImpl.call(this, $formCont);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _renderPageTitle: function ($cont) {
        // TODO Milen: For Petar/Iliya: Why there is so complicated structure
        $('<h2 class="secureonly"></h2>')
            .setTextById("activation_title")
            .appendTo($cont);
        var $p = $('<p class="p secureonly">').appendTo($cont);
        $('<span></span>')
            .setTextById("activation_subtitle")
            .appendTo($p);
        // See FF-4442
        $('<p></p>')
            .setTextById("activation_subtitle2")
            .appendTo($cont);
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _setControlsState: function (onDisable, notForSubmit) {
        FrameActivate.superclass._setControlsState.call(this, onDisable, notForSubmit);

        // update input
        this._getFormControl(FrameWithForm.FIELD_CODE, true).attr("disabled", onDisable);
        // update buttons
        var $btnSubmit = this._getBtn(BTN_ACTIVATE_ID);
        if(!notForSubmit)
            $btnSubmit.button(onDisable ? 'loading' : 'reset');
        $btnSubmit.attr("disabled", onDisable);

        this._updateSendLogLinkState(!onDisable);
    },

    /* Private method */

    /**
     * The Device Activation page is rendered here. label_remote_access and label_remote_activate
     * can be different depending on the OS. In admin-en.xlf the different field_domain_info options of these variables are shown.
     * @param {jQuery} $cont
     * @private
     */
    _renderPageHome: function ($cont) {
        var $comp = $('<div id="pageHome"></div>');
        $comp.appendTo("body");
        var opts = {
            "isSpec": true
        };
        var remoteAccess = QnLocale.getStr("label_remote_access", opts);
        var remoteActivate = QnLocale.getStr("label_remote_activate", opts);
        var optsHomePage = {
            "listArg": {
                "label_remote_access": remoteAccess,
                "label_remote_activate": remoteActivate
            }
        };
        $comp.setTextById("welcome_page", optsHomePage);
        QnLocale.setTextFromAttr($comp);
        $comp.appendTo($cont);

        var self = this;
        $comp.find(".buttonRegister")
            .on("click", function (e) {
                self._handleAction("openRegister", e);
        });
        $comp.find("#actionActivate").on("click", function (e) {
            self._handleAction("openActivate", e);
        });

        // Hide initially both pages until languages are loaded
        $comp.hide();
    },

    /**
     *
     * @private
     */
    _onRegister: function () {
        var params = {
            "type" : "canvas"
        };
        var registerUrl = "https://share.fileflex.com/fbweb/app/public/view/register";
        var deviceScheme = getGlobal(KEY_GLOBAL_DEVICE_SCHEME);
        if(deviceScheme)
            params["alias"] = deviceScheme;

        registerUrl = addParams(registerUrl, params);

        window.open(registerUrl,'_blank');
    },

    /**
     *
     * @private
     */
    _onActivate: function () {
        var activateUrl = window.location.href;
        // var deviceScheme = getGlobal(KEY_GLOBAL_DEVICE_SCHEME);
        // if (deviceScheme)
        //     activateUrl = addParams(activateUrl, {"alias" : deviceScheme});

        activateUrl += "#activate";

        window.open(activateUrl, '_self');
    },

    /**
     *
     * @private
     */
    _startActivate: function () {
        // enter a username into the edit box
        var username = getGlobal(KEY_GLOBAL_USERNAME);
        if (username) {
            log("found username: " + username);
            this._getFormControl(FrameWithForm.FIELD_USERNAME).val(username);
        }

        // set the domain if passive provided one (set in the settings page)
        var defDomain =  getGlobal(KEY_GLOBAL_DEFAULT_DOMAIN);
        if (defDomain) {
            log("found default domain: " + defDomain);
            this._getFormControl(FrameWithForm.FIELD_DOMAIN).val(defDomain);
        }

        // enter a domain into the edit box
        var domain = getGlobal(KEY_GLOBAL_DOMAIN);
        if (domain) {
            log("found domain: " + domain);
            this._getFormControl(FrameWithForm.FIELD_DOMAIN).val(domain);
        }

        // enter a code into the edit box
        var code = getGlobal(KEY_GLOBAL_CODE_ACTIVATE);
        if (code) {
            log("found activation code: " + code);
            this._getFormControl(FrameWithForm.FIELD_CODE).val(code);
        }

        var retryLink = getGlobal(KEY_GLOBAL_RETRY_LINK);
        if (retryLink)
            log("found retryLink: " + retryLink);

        var cancelLink = getGlobal(KEY_GLOBAL_CANCEL_LINK);
        if (cancelLink)
            log("found cancelLink: " + cancelLink);
    },

    /**
     * Handle successful response
     *
     * @param {String} redirectURL
     * @private
     */
    _handleActivationSuccess: function (redirectURL) {
        // success
        log("activation successful");
        // indicate success visually
        showMessage(ERROR_OK, "activate_error_ok");
        this._getFormControl(FrameWithForm.FIELD_CODE).val("");

        // if a redirect URL was provided by passive (via the server) we use it to send the user back immediately
        if (redirectURL !== null) {
            // if there is a redirect URL, we immediately redirect the user
            log("redirectURL found: " + redirectURL);
            window.location.href = redirectURL;
        } else {
            log("no redirect URL was found");
        }
    }

});

/**
 *
 * @type {string}
 * @constant
 */
FrameActivate.VID = "FrameActivate";

/**
 *
 * @type {string}
 * @private
 * @constant
 */
FrameActivate._PAGE_ACTIVATE = "activate";
/**
 *
 * @type {string}
 * @private
 * @constant
 */
FrameActivate._PAGE_HOME = "home";
