/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 1.3.2018
 * Time: 10:53
 * Author Petar Vanchev (petarv@qnext.com)
 */

/**
 *
 * @param {jQuery} $container
 * @param {Object} options
 * @param {Boolean} doNotRequestStatus
 * @constructor
 */
var FrameNetworkSettings = function ($container, options, doNotRequestStatus) {
    FrameWithForm.call(this, FrameNetworkSettings.VID, $container, options, doNotRequestStatus);
};

QnUtil.inherits(FrameNetworkSettings, FrameWithForm);
QnUtil.extend(FrameNetworkSettings.prototype, {

    /* Public method */

    updateFrame: function() {
        setTextContext("field_networkBrowse_info");
        setTextContext("field_hostName_info");
        setTextContext("field_defaultDomain_info");

        this._updateNetwork();
        this._updateField(FrameNetworkSettings.FIELD_HOST_NAME);
        this._updateField(FrameNetworkSettings.FIELD_DEFAULT_DOMAIN);
        // Hide "Cancel" buttons for browsers
        this._toggleBtn(BTN_CANCEL_ID);

        FrameNetworkSettings.superclass.updateFrame.apply(this);

        this._updateSubmitBtnState();
    },


    /* Protected method */

    /**
     * @inheritDoc
     * @protected
     */
    _initContentImpl: function ($cont) {
        var  $formCont = $('<div id="pageNetworkSettings"></div>').appendTo($cont);
        FrameNetworkSettings.superclass._initContentImpl.call(this, $formCont);
    },

    _getFieldIdList: function () {
        return [FrameNetworkSettings.CHECKBOX_NETWORK_BROWSE, FrameNetworkSettings.FIELD_HOST_NAME,
            FrameNetworkSettings.FIELD_DEFAULT_DOMAIN];
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getBtnIdList: function () {
        return [BTN_SAVE_ID, BTN_CANCEL_ID];
    },

    /**
     *
     * @return {string}
     * @protected
     */
    _getControlIdPrefix: function () {
        return "device-";
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getFieldParams: function (fieldName) {
        var params = {
            "isRequired": this._isRequired(fieldName),
            "isReadOnly": this._isReadOnly(fieldName),
            "isChecked": this._isChecked(fieldName),
            "inputType": "text"
        };

        switch (fieldName) {
            case FrameNetworkSettings.CHECKBOX_NETWORK_BROWSE:
            case FrameNetworkSettings.FIELD_HOST_NAME:
            case FrameNetworkSettings.FIELD_DEFAULT_DOMAIN:
                if(fieldName === FrameNetworkSettings.CHECKBOX_NETWORK_BROWSE)
                    params["inputType"] = "checkbox";
                params["ttContClass"] = fieldName.capitalize();
                params["ttPointerClass"] = params["ttContClass"];
                break;
            default:
                return FrameNetworkSettings.superclass._getFieldParams.call(this, fieldName);
        }

        return params;
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _renderPageTitle: function ($cont) {
        var $desc = $('<h2 class="secureonly"></h2>')
            .setTextById("network_settings_title");
        $desc.appendTo($cont);
    },

    /**
     * Check did filed must be mark as isRequired or not
     *
     * @param {String} fieldId
     * @return {Boolean} always valid. Default is true
     * @protected
     */
    _isRequired: function (fieldId) {
        switch (fieldId) {
            case FrameNetworkSettings.FIELD_DEFAULT_DOMAIN:
            case FrameNetworkSettings.FIELD_HOST_NAME:
                return false;
            default:
                return true;
        }
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _fillSubmitParams: function (params) {
        params["cmd"] = "set-global";
        params[KEY_GLOBAL_NETWORK_ENABLED] = this._getFormControl(FrameNetworkSettings.CHECKBOX_NETWORK_BROWSE).is(':checked');
        params[KEY_GLOBAL_HOST_OVERRIDE] = this._getFormControl(FrameNetworkSettings.FIELD_HOST_NAME).val().trim();
        params[KEY_GLOBAL_DEFAULT_DOMAIN] = this._getFormControl(FrameNetworkSettings.FIELD_DEFAULT_DOMAIN).val().trim();
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _getRequestUrl: function () {
        return "/secure/";
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleGuiEvent: function ($comp, event) {
        this._updateSubmitBtnState();
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitImpl: function (event) {
        var hasChange = this._hasChanges();
        if(hasChange)
            hasChange = confirm(QnLocale.getStr("device_network_settings_save_confirm"));
        return hasChange;
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitSuccessImpl: function (resp, requestParams) {
        if (resp.result) {
            var redirectURL = resp["redirectURL"];
            log("Device Network  settings update successful");
            // indicate success visually
            showMessage(ERROR_OK, "device_network_settings_success");

            // if a redirect URL was provided by passive (via the server) we use it to send the user back immediately
            if (redirectURL != null) {
                // if there is a redirect URL, we immediately redirect the user
                log("redirectURL found: " + redirectURL);
                window.location.href = redirectURL;
            } else {
                log("no redirect URL was found");
            }

        } else {
            log("Device Network  settings update failure");
        }
    },

    /**
     *
     * @inheritDoc
     * @protected
     */
    _handleSubmitFailImpl: function (xhr, err) {
        if (xhr.responseText) {
            try {
                var response = JSON.parse(xhr.responseText);
                var error = response["error"];
                log("Device Network settings update failed error code: " + error["code"]);
            } catch (e) {
                log("Device Network  settings update failed: " + e);
            }
        }
    },

    /**
     * Enable/Disable form controls (inputs, buttons etc..)
     *
     * @param {Boolean} [onDisable]
     * @param {Boolean} [notForSubmit]
     * @protected
     */
    _setControlsState: function (onDisable, notForSubmit) {
        // update input
        this._getFormControl(FrameNetworkSettings.CHECKBOX_NETWORK_BROWSE, true).attr("disabled", onDisable);
        this._getFormControl(FrameNetworkSettings.FIELD_HOST_NAME, true).attr("disabled", onDisable);
        this._getFormControl(FrameNetworkSettings.FIELD_DEFAULT_DOMAIN, true).attr("disabled", onDisable);
        // update buttons
        var $btnSubmit = this._getBtn(BTN_SAVE_ID);
        if(!notForSubmit)
            $btnSubmit.button(onDisable ? 'loading' : 'reset');
        $btnSubmit.attr("disabled", onDisable);
    },


    /* Private method */

    /**
     * Check did GUI is changed from default values that was set on init
     *
     * @return {Boolean} always valid
     * @private
     */
    _hasChanges: function () {
        var hasChanges = false;

        var fieldIdList = this._getFieldIdList();
        for (var i = 0; i < fieldIdList.length; i++) {
            var id = fieldIdList[i];
            var $comp = this._getFormControl(id, true);

            switch (id) {
                case FrameNetworkSettings.CHECKBOX_NETWORK_BROWSE:
                    hasChanges = $comp.is(':checked') !== getGlobal(KEY_GLOBAL_NETWORK_ENABLED);
                    break;
                case FrameNetworkSettings.FIELD_HOST_NAME:
                    hasChanges = $comp.val().trim() !== getGlobal(KEY_GLOBAL_HOST_OVERRIDE);
                    break;
                case FrameNetworkSettings.FIELD_DEFAULT_DOMAIN:
                    hasChanges = $comp.val().trim() !== getGlobal(KEY_GLOBAL_DEFAULT_DOMAIN);
                    break;
            }

            if(hasChanges)
                break;
        }

        return hasChanges;
    },

    /**
     * Enable/Disable save button
     *
     * @private
     */
    _updateSubmitBtnState: function () {
        this._getBtn(BTN_SAVE_ID).prop("disabled", !this._hasChanges());
    },

    /**
     * Update network browsing checkbox state (checked/unchecked)
     *
     * @private
     */
    _updateNetwork: function () {
        var isChecked = getGlobal(KEY_GLOBAL_NETWORK_ENABLED);
        var $comp = this._getFormControl(FrameNetworkSettings.CHECKBOX_NETWORK_BROWSE, true);
        if($comp.length > 0)
            $comp.prop("checked", isChecked);
    },

    /**
     * Update filed value
     *
     * @param {String} fieldId
     * @private
     */
    _updateField: function (fieldId) {
        var statusValue;
        switch (fieldId) {
            case FrameNetworkSettings.FIELD_HOST_NAME:
                statusValue = getGlobal(KEY_GLOBAL_HOST_OVERRIDE);
                break;
            case FrameNetworkSettings.FIELD_DEFAULT_DOMAIN:
                statusValue = getGlobal(KEY_GLOBAL_DEFAULT_DOMAIN);
                break;
        }

        var $comp = this._getFormControl(fieldId, true);

        if($comp.length > 0 && QnUtil.isDefined(statusValue) && statusValue !== $comp.val())
            $comp.val(statusValue);
    }

});

/**
 *
 * @type {string}
 * @constant
 */
FrameNetworkSettings.VID = "FrameNetworkSettings";

/**
 *
 * @type {string}
 * @constant
 */
FrameNetworkSettings.CHECKBOX_NETWORK_BROWSE = "networkBrowse";
/**
 *
 * @type {string}
 * @constant
 */
FrameNetworkSettings.FIELD_HOST_NAME = "hostName";
/**
 *
 * @type {string}
 * @constant
 */
FrameNetworkSettings.FIELD_DEFAULT_DOMAIN = "defaultDomain";