/**
 * Copyright Qnext Inc. 2002-2018
 * http://www.qnext.com
 * Date: 8.2.2018
 * Time: 15:48
 * Author Petar Vanchev (petarv@qnext.com)
 * Author Milen Dimitrov (milend@qnext.com)
 */

/**
 * Form that has submit form
 *
 * @param {String} vid
 * @param {jQuery} $container when not provided frame will be added to the body element
 * @param {Object} [options] see {@link FrameMain} doc
 * @param {Boolean} [doNotRequestStatus]
 * @constructor
 */
var FrameWithForm = function (vid, $container, options, doNotRequestStatus) {
    /**
     *
     * @type {jQuery}
     * @private
     */
    this._$form = null;
    /**
     * Determine did frame can submit data to server or not. Default is true. Will be updated only from {@link #_checkCanSubmit}
     *
     * @type {boolean}
     * @private
     */
    this._canSubmit = true;

    FrameMain.call(this, vid, $container, options, doNotRequestStatus);
};

QnUtil.inherits(FrameWithForm, FrameMain);
QnUtil.extend(FrameWithForm.prototype, {

    /* Public method */

    updateFrame: function (isFromStatus) {
        FrameWithForm.superclass.updateFrame.call(this, isFromStatus);

        /* Info windows */
        this._addFocusHandlers();

        // check did form can be submitted
        this._checkCanSubmit();
    },

    /* Protected method */

    /**
     * @inheritDoc
     * @protected
     */
    _initContentImpl: function ($cont) {
        this._renderPageTitle($cont);
        this._initFormContent($cont);
    },

    /**
     * @param {jQuery} $cont
     * @protected
     */
    _renderPageTitle: function ($cont) {
    },

    /**
     *
     * @param {jQuery} $container
     * @protected
     */
    _renderFormImpl: function ($container) {
    },

    /**
     *
     * @return {Boolean}
     * @protected
     */
    _checkCanSubmitImpl: function () {
        return true;
    },

    /**
     *
     * @param {Event} event
     * @return {Boolean}
     * @protected
     */
    _handleSubmitImpl: function (event) {
        return true;
    },


    /**
     *
     * @param resp
     * @param requestParams
     * @protected
     */
    _handleSubmitSuccessImpl: function (resp, requestParams) {
    },

    /**
     *
     * @param xhr
     * @param err
     * @protected
     */
    _handleSubmitFailImpl: function (xhr, err) {
    },

    /**
     * !!! Do not override use {@link #_handleActionImpl} instead
     * @param {String} action
     * @param {Event} event
     * @param {Boolean} [noPrevent]
     * @protected
     * @readonly
     */
    _handleAction: function (action, event, noPrevent) {
        if (!noPrevent)
            event.preventDefault();

        var $targetComp = $(event.target);
        // prevent action handling over disabled component
        if (!$targetComp.hasClass("disabled") && !$targetComp.prop("disabled"))
            this._handleActionImpl(action, event);
    },

    /**
     *
     * @param {String} action
     * @param {Event} event
     * @protected
     */
    _handleActionImpl: function (action, event) {
        switch (action) {
            case "cancel":
                this._handleCancel();
                break;
            case "sendLogs":
                this._handleSendLogs();
                break;
            default:
                log("Unknown action: " + action, true);
        }
    },

    /**
     * Enable/Disable form controls (inputs, buttons etc..)
     *
     * @param {Boolean} [onDisable]
     * @param {Boolean} [notForSubmit]
     * @protected
     */
    _setControlsState: function (onDisable, notForSubmit) {
    },

    /**
     * Get list with field id that what will be render inside form
     *
     * @return {String[]} always valid but can be empty
     * @protected
     */
    _getFieldIdList: function () {
        return [];
    },

    /**
     * Get list with button id that what will be render inside form
     *
     * @return {String[]} always valid but can be empty
     * @protected
     */
    _getBtnIdList: function () {
        return [];
    },

    /**
     * Get button comp by id
     *
     * @param {String} btnId
     * @return {jQuery}
     * @protected
     */
    _getBtn: function (btnId) {
        return $("#" + this._genControlId(btnId));
    },

    /**
     * Toggle button by id
     *
     * @param {String} btnId
     * @protected
     */
    _toggleBtn: function (btnId) {
        toggleButton(this._genControlId(btnId));
    },

    /**
     *
     *
     * @param {String} name
     * @param {Boolean} [isById] determine how to select field by id or class. Default is false (by class)
     * @return {jQuery}
     * @protected
     */
    _getFormControl: function (name, isById) {
        var $control;

        if (isById) {
            var fieldId = this._genControlId(name);
            $control = getCompById(fieldId);
        } else {
            var fieldClass = this._genControlClass(name);
            $control = getComp(fieldClass);
        }

        return $control;
    },

    /**
     * Fill submit request params
     *
     * @param {Object} params
     * @protected
     */
    _fillSubmitParams: function (params) {
    },

    /**
     * Get ajax request url
     *
     * @return {String} relative url path. Default is "/" (root)
     * @protected
     */
    _getRequestUrl: function () {
        return "/";
    },

    /**
     * Check did filed must be mark as readonly or not
     *
     * @param {String} fieldId
     * @return {Boolean} always valid
     * @protected
     */
    _isReadOnly: function (fieldId) {
        return false;
    },

    /**
     * Check did control must be checked or not
     *
     * @param {String} fieldId
     * @return {Boolean}
     * @protected
     */
    _isChecked: function (fieldId) {
        return false;
    },

    /**
     * Check did filed must be mark as isRequired or not
     *
     * @param {String} fieldId
     * @return {Boolean} always valid. Default is true
     * @protected
     */
    _isRequired: function (fieldId) {
        return true;
    },

    // TODO Petar: make it to work similar to _addFocusHandlers
    // _setTextAndImageContext: function () {
    //     var list = this._getFieldIdList();
    //     for(var i = 0; i < list.length; i++) {
    //         var name = list[i];
    //         var hasImage = name !== FrameWithForm.FIELD_CODE;
    //         var compClass = this._genControlClass(name);
    //         setTextContext(compClass);
    //         if(hasImage) {
    //             // make first latter of name to be Capital
    //             name = name.capitalize();
    //             setImageContext("scr" + name);
    //         }
    //     }
    // },

    /**
     * Fill params needed to be render control (filed, checkbox ...) and its label and tooltip
     *
     * @param {String} fieldName
     * @return {Object} can be null if fieldName is not found
     * @protected
     */
    _getFieldParams: function (fieldName) {
        return null;
    },

    /**
     *
     * @return {string} always valid can be empty
     * @protected
     */
    _getControlIdPrefix: function () {
        return "";
    },

    /**
     * Handle GUI events fired from controls (events: keyup, change)
     *
     * @param {jQuery} $comp
     * @param {Event} event
     * @protected
     */
    _handleGuiEvent: function ($comp, event) {
    },

    /**
     * On "Cancel" button action
     * @protected
     */
    _handleCancel: function () {
        onReturnToApp();
    },


    /* Private method */

    /**
     *
     * @param {String} fieldName
     * @private
     */
    _addFieldToForm: function (fieldName) {
        var self = this;
        var params = this._getFieldParams(fieldName);

        if (params) {
            var inputType = params["inputType"];
            var isRequired = params["isRequired"];
            var isReadOnly = params["isReadOnly"];
            var isChecked = params["isChecked"];
            var groupClass = this._genControlClass(fieldName);


            var $comp = this.createCont().addClass("form-group").appendTo(this._$form);
            // render tooltip
            this._renderTooltip($comp, fieldName, params);
            // render filed and label for it
            var attributes = {
                "type": inputType,
                "id": this._genControlId(fieldName)
            };
            var $control;
            if (inputType === "checkbox" || inputType === "radio") {
                var groupName = params["groupName"];
                if (groupName)
                    attributes["name"] = groupName;

                $control = $('<input class="form-control"/>')
                    .attr(attributes)
                    .addClass("form-control " + groupClass)
                    .prop("checked", isChecked)
                    .appendTo($comp);
                $('<span></span>')
                    .addClass("control-label")
                    .setTextById("field_" + fieldName + "_header", {isSpec: true})
                    .appendTo($comp);
                $('<br/>').appendTo($comp);
            } else {

                this.createCont()
                    .addClass("control-label")
                    .setTextById("field_" + fieldName + "_header", {isSpec: true})
                    .appendTo($comp);
                $control = $('<input />')
                    .attr(attributes)
                    // Optional for some inputs like checkboxes
                    .setTooltip("field_" + fieldName + "_placeholder", {isSpec: true, isOpt: true})
                    .addClass("form-control " + groupClass)
                    .prop("required", isRequired)
                    .prop("readonly", isReadOnly)
                    .appendTo($comp);
            }

            switch (inputType) {
                case "text":
                case "password":
                    $control.on("keyup", function (e) {
                        self._handleGuiEvent($(this), e);
                    });
                    break;
                case "checkbox":
                case "radio":
                    $control.on("change", function (e) {
                        self._handleGuiEvent($(this), e);
                    });
                    break;
            }
        }
    },

    /**
     * Render tooltip
     *
     * @param {jQuery} $container
     * @param {String} fieldName component id for witch this tooltip will be
     * @param {Object} params
     * @private
     */
    _renderTooltip: function ($container, fieldName, params) {
        // TODO Milen: For Petar: To Use this: but
        // 1. appendTo, not prependTo is used
        // 2. two different classes: ttPointerClass and ttContClass
        // this._addTooltip($container, params["ttPointerClass"], "field_" + fieldName);

        $('<div class="tooltipCont"><div class="tooltipInner"></div></div>')
            .addClass("tooltip" + params["ttContClass"])
            .appendTo($container);
        var $ttInner = $container.find(".tooltipInner");

        // inner tooltip elements
        var ttPointerClass = params["ttPointerClass"];
        this.createCont()
            .addClass("tooltipPointer" + ttPointerClass)
            .appendTo($ttInner);
        this.createCont()
            .addClass("tooltipContTitle")
            .setTextById("field_" + fieldName + "_placeholder", {isSpec: true})
            .prop("required", !!params["isRequired"])
            .appendTo($ttInner);
        $('<hr/>').appendTo($ttInner);
        this.createCont()
            .addClass("tooltipContDesc")
            .attr("id", "field_" + fieldName + "_info")
            .appendTo($ttInner);
        // TODO Petar: make img tag to be optional
        $('<img class="scr" />')
            .attr("id", "scr" + ttPointerClass)
            .appendTo($ttInner);
    },

    /**
     * Add focus handle to form field. First field will be default focus component
     *
     * @private
     */
    _addFocusHandlers: function () {
        var list = this._getFieldIdList();
        var isFocused = false;
        for (var i = 0; i < list.length; i++) {
            var compId = this._genControlId(list[i]);
            if (!isFocused && !getCompById(compId).prop("readonly")) {
                isFocused = true;
                addFocusHandler(compId, true);
            } else {
                addFocusHandler(compId);
            }
        }
    },

    /**
     * Generate comp id
     *
     * @param {String } name
     * @return {String}
     * @private
     */
    _genControlId: function (name) {
        var controlId = "";
        if (name === FrameWithForm.FIELD_USERNAME)
            name = "name";

        if (name)
            controlId = this._getControlIdPrefix() + name;

        return controlId;
    },

    /**
     * Generate comp class
     *
     * @param {String} name
     * @return {String}
     * @private
     */
    _genControlClass: function (name) {
        var controlClass = "";

        if (name)
            controlClass = "field-" + this._genControlId(name);

        return controlClass;
    },

    /**
     *
     * @param {jQuery} $cont
     * @private
     */
    _initFormContent: function ($cont) {
        // add form message container
        $('<div id="updatemsgForm"></div>').appendTo($cont);
        // add form wrapper
        var $formWrap = $('<div class="formWraper">' +
            '<div class="secureonly formCont">' +
            '</div>' +
            '</div>').appendTo($cont);
        var $formCont = $formWrap.find(".formCont");
        // render form and buttons
        this._renderForm($formCont);
        // add help section
        this._renderHelpSection($formCont);
    },

    /**
     * Add button with given id to {@link FrameWithForm#_$form}
     *
     * @param {String} btnId can be one of {@link BTN_ACTIVATE_ID} ...
     * @private
     */
    _addButtonToForm: function (btnId) {
        var $btnComp = $('<button></button>').attr("id", this._genControlId(btnId));
        var label, type, action;
        switch (btnId) {
            case BTN_CANCEL_ID:
                label = "cancel";
                action = "cancel";
                $btnComp.addClass("buttonCancel");
                break;
            case BTN_ACTIVATE_ID:
                type = "submit";
                label = "activate";
                break;
            case BTN_SAVE_ID:
                type = "submit";
                label = "save";
                break;
            default:
                return;
        }

        $btnComp.attr("type", type ? type : "button");
        if (label) {
            $btnComp.setTextById(label + "_button");
            // set loading text fix for https://jira2.cnexus.com/browse/FF-6540
            var btnLoadingStrId = label + "_button_loading";
            if (QnLocale.hasStr(btnLoadingStrId))
                $btnComp.attr("data-loading-text", QnLocale.getStr(btnLoadingStrId));
        }

        $btnComp.appendTo(this._$form);

        if (action) {
            var self = this;
            $btnComp.on("click", function (e) {
                self._handleAction(action, e);
            });
        }
    },

    /**
     *
     * @param {jQuery} $cont
     * @private
     */
    _renderForm: function ($cont) {
        this._$form = $('<form class="form-signin" role="form"></form>').appendTo($cont);
        var i;
        // add fields to form
        var listFldId = this._getFieldIdList();
        for (i = 0; i < listFldId.length; i++) {
            this._addFieldToForm(listFldId[i]);
        }

        this._renderFormImpl(this._$form);

        // add buttons to form
        var listBtnId = this._getBtnIdList();
        for (i = 0; i < listBtnId.length; i++) {
            this._addButtonToForm(listBtnId[i]);
        }

        // add submit handing of form
        var self = this;
        this._$form.submit(function (event) {
            self._handleSubmit(event);
        });
    },

    /**
     *
     * @param event
     * @private
     */
    _handleSubmit: function (event) {
        event.preventDefault();
        if (this._canSubmit) {
            if (this._handleSubmitImpl(event)) {
                // disable form controls and enable them on response or error
                this._setControlsState(true);
                // proceed with the activation attempt
                this._psvSubmit();
            }
        }
    },

    /**
     * Request register a code or update user settings
     *
     * @private
     * @readonly
     */
    _psvSubmit: function () {
        var self = this;
        var params = {};
        this._fillSubmitParams(params);

        var jqXHR = $.ajax({
            method: "GET",
            url: self._getRequestUrl(),
            data: params,
            dataType: "json"
        });

        var code;
        jqXHR.done(function (resp) {
            self._handleSubmitSuccessImpl(resp, params);
            self._setControlsState(false);

            if (!resp.result) {
                code = resp.code;
                var respMsg = resp.msg;
                if (!code && respMsg)
                    code = translateMsgToErrorCode(respMsg);
                translateErrorCode(code, self._getMainCont().find("#updatemsgForm"));
            } else {
                self.getStatus();
            }
        }).fail(function (xhr, err) {
            self._handleSubmitFailImpl(xhr, err);
            if (xhr.responseText) {
                try {
                    var response = JSON.parse(xhr.responseText);
                    var error = response["error"];
                    code = error["code"];
                    log("Form submit failed error code: " + code);
                } catch (e) {
                    log("Form submit failed: " + e);
                    code = ERROR_LOGIN;
                }
                log("Invalid login credentials");
            } else {
                code = getError(xhr, err);
            }
            self._setControlsState(false);
            translateErrorCode(code, self._getMainCont().find("#updatemsgForm"));
        });
    },

    /**
     *
     * @private
     */
    _handleSendLogs: function () {
        var code = this._getFormControl(FrameWithForm.FIELD_CODE).val();
        // code param is obligatory
        if (code) {
            var sendLogsUrl = "/send-logs.html";

            var params = {};
            params[KEY_GLOBAL_CODE_ACTIVATE] = code;

            // treat 'meta' param differently as it should be an object
            var meta = getGlobal(KEY_GLOBAL_META);
            meta = meta ? JSON.parse(meta) : {};
            meta["isActivate"] = true;
            meta = JSON.stringify(meta);
            params[KEY_GLOBAL_META] = meta;

            // add some optional params

            var device = getGlobal(KEY_GLOBAL_CLIENT_DEVICE);
            if (device)
                params[KEY_GLOBAL_CLIENT_DEVICE] = device;

            var mspId = getGlobal(KEY_GLOBAL_MSP_ID);
            if (mspId)
                params[KEY_GLOBAL_MSP_ID] = mspId;

            var uid = getGlobal(KEY_GLOBAL_UID);
            if (uid)
                params[KEY_GLOBAL_UID] = uid;

            sendLogsUrl = addParams(sendLogsUrl, params);

            window.open(sendLogsUrl, '_blank');
        } else {
            // TODO Rumen - show info/warning or something
            // for now do nothing
        }
    },

    /**
     * Check did form can be submitted
     *
     * @private
     */
    _checkCanSubmit: function () {
        this._canSubmit = this._checkCanSubmitImpl();
        if (this._canSubmit) {
            this._setControlsState(false, true);
        } else {
            this._setControlsState(true, true);
        }
    }
});

/**
 *
 * @type {string}
 * @constant
 */
FrameWithForm.FIELD_USERNAME = "username";
/**
 *
 * @type {string}
 * @constant
 */
FrameWithForm.FIELD_PASSWORD = "password";
/**
 *
 * @type {string}
 * @constant
 */
FrameWithForm.FIELD_DOMAIN = "domain";
/**
 *
 * @type {string}
 * @constant
 */
FrameWithForm.FIELD_CODE = "code";